/* $Id: MSGLIB.H 1.8 1997/02/12 06:51:40 ska Exp ska $
   $Locker: ska $	$Name:  $	$State: Exp $

	Global message handling definitions.

   $Log: MSGLIB.H $
   Revision 1.8  1997/02/12 06:51:40  ska
   chg: in Micro-C code: the way to justify the output, from
   			%d ==    <-1>				%d ==    <-1>
   			%5d ==   <-   1>			%5d ==   <   -1>
   	from	%-5d ==  <-1   >	to		%-5d ==  <-1   >
   			%05d ==  <-0001>			%05d ==  <-0001>
   			%-05d == <-1000>			%-05d == <-1   >
   	also: the '0' filler is used for numerics only.
   	by doing so, the Micro-C code no longer depends on printf(), instead
   	it uses either ltoa() or itoa() to create ASCIZ from numericals.
   add: in Micro-C code: If LSIZE is defined, treat it as LOMGMATH array size
   	If not, assume 4 bytes
   chg: in Micro-C code: use ltoa() instead of the work-around for long values
   bugfix: in Micro-C code: %-#%s, %-#%x, %-#%-%R was ignored
   add: raw data enclosure: %-%W
   chg: ANSI enclosure in strings to: %-%A
   chg: E_hlpScreen is not automatically referenced
   chg: MSG_DCL() macro has four parameters
   bugfix: error()/fatal() will terminate program if noiseLevel is denied
   add: smessage() / ENoise_screen
   chg: the functions fatal(), error(), warning(),& informative()
   	don't append '.' only a '\n'

   Revision 1.7  1995/08/30 07:03:19  ska
   Release 4.

   Revision 1.6  1995/08/11 06:49:47  ska
   add: "internal failure" calling function.

   Revision 1.5  1995/06/28 15:36:58  ska
   release 3 of message library.

   Revision 1.4  1995/05/26 22:53:58  ska
   processed via msgComp while update with msg file

   Revision 1.3  1995/05/26 20:29:24  ska
   add: access to application's name.
   add: prototype of fputmc().

   Revision 1.2  1995/04/21 04:35:46  ska
   version 1.0

   Revision 1.1  1995/04/21 04:11:29  ska
   Initial revision

*/

#ifndef __MSGLIB_H
#define __MSGLIB_H

#include <stdio.h>
#include <portable.h>


/*
 *	MSGID is the black-box type to hold the identifier of a message
 */
#ifdef _MICROC_	/* does not support typedef's */
#define MSGID unsigned char*
#else
typedef const unsigned char *MSGID;
#endif

/*
 *	DCL_MSG declares one MeSsaGe IDentifier;
 */
#define DCL_MSG(msg,group,id) extern const unsigned char msg[];

#define msgLock(msg) ((char*)msg)
#define msgRetreive(msg) ((char*)msg)
#define msgUnlock(msg)
#define msgUnlockString(msg)
/* this needs the one-byte alignment within the error definition files */
#define msgErrorNumber(msg) ((msg)[-1]) 
#define iFailure(nr) fatal(E_internalFailure, nr)
/* msgInit() is a function */

#define ANSI_TEST 0		/* check if output stream is connected to a CON device */
#define ANSI_DENY 1		/* do no send ANSI sequences into the output stream */
#define ANSI_ALLOW 2	/* do not check for CON device, send sequences */

#define NOISE_TEST 2	/* check if stdin is redirected, before emitting interactive messages.
						   skip if not interactive message */
#define NOISE_DENY 1	/* skip messages t all */
#define NOISE_ALLOW 0	/* emit messges */
#define NOISE_ERR (-1)	/* error, returned by getNoiseLevel() on error */

#define ENoise_interactive 0	/* must be number zero !!! */
#define ENoise_informative 1
#define ENoise_fatal 2
#define ENoise_error 3
#define ENoise_warning 4
#define ENoise_none 5
#define ENoise_screen 6
#define	ENoise_Nr 7				/* number of noise levels */



void msgInit(void);
/* Initialize the message system.

   Currently:
   	+ Check for ANSI and set the ANSI test flag to ANSI_TEST or ANSI_DENY.
   	+ Check for redirected stdin and set the interactive test flag to
   	  NOISE_ALLOW or NOISE_DENY.
   	+ Determine the application's name.
*/

#ifdef _MICROC_
#define msgErrFct(fct) _msgErrFct(&(fct))
void _msgErrFct(int fct);
#else
void msgErrFct(void (*fct)(void));
#endif
/* Set a function, which will be executed right before the exit()-call,
   after an error() or fatal() message has been emitted
*/

char *appName(void);
/* Return the name of the current process. This name is upper-cased
   and without drive, path, and extension.
*/


int fputmc(int c, int rep, FILE *fout);
/* Dump the character rep times into the stream fout.

	Return:		 0: OK
				!0: failure
*/


int skprintf(unsigned char *poi, unsigned len, char *fmt, const unsigned *args);
/* input:	poi: output stream; must not be NULL!
   			len: length of the output buffer:
   				== 0 => output buffer is stream (poi is FILE*)
   				else => output buffer is memory (terminating NUL is added)
   					the buffer is filled with up to (len-1) bytes
   					is used to indicate, how much space is left in the buffer
   			fmt: enhanced format string
   			args: pointer to the first argument

   output: number of written characters or EOF on failure

   This function uses the isatty() function on the stream fout to
   determine, if to emit the ANSI enclosures or not. That behaviour is
   controlled by the ANSI test flag. When (len != 0) ANSI is always
   disabled.

*/

#ifndef _MICROC_
int fd_fprintf(FILE *fp, char *fmt, ...);
int fd_sprintf(char *buf, char *fmt, ...);
#else
register int fd_fprintf(FILE *fp, char *fmt);
register int fd_sprintf(char *buf, char *fmt);
#endif
/* Wrapper for skprintf().

	These functions allow to call skprintf() with the same 
	calling interface than standard fprintf() and sprintf().
*/


void msgSetAnsiFlag(int flag);
/* Function to set the ANSI test flag. 

   Should be called before msgInit() to skip the check, if an ANSI
   driver is present. This test will set the flag to ANSI_TEST if a
   driver is present and to ANSI_DENY if none is found.

   In the functions using this flag, ANSI_TEST means, that ANSI will
   be used, if the file stream is connected to a console (CON:).

   May be called anytime the application wants to alter the ANSI test
   behaviour of skprintf(). To retrieve the current state of the flg
   use getAnsiTest().

   flag should be one of ANSI_????, otherwise the behaviour of all
   functions using this flag is undefined.

   Initial setting of the flag: ANSI_TEST.

*/

int msgGetAnsiFlag(void);
/* Function retrieves the current state of the ANSI test flag
*/

int checkANSI(void);
/* Return whether or not an ANSI driver is loaded.
*/


void setNoiseLevel(unsigned noiseLevel, int flag);
/* Function to set the allow/disallow flag for each noise level.

   May be called anytime the application wants to alter the interactive
   test behaviour of the interactive() function. To retrieve the current
   state of the flag use getNoiseLevel().

   flag should be one of NOISE_???, otherwise the behaviour of all
   functions using this flag is undefined.

   Initial setting is NOISE_ALLOW.

	For ENoise_interactive only:

   Should be called after msgInit() to ensure, that the test is
   performed anytime an interactive message is emitted.

   Initial setting of the ENoise_interactive flag: NOISE_TEST, but is
   altered to either NOISE_DENY or NOISE_ALLOW by the msgInit()
   function.

*/

int getNoiseLevel(unsigned noiseLevel);
/* Function retrieves the current state of the allow/disallow flag
   of the specified noise level.
*/



/* Message emitting functions:

   hlpScreen:
   	Emit the message E_hlpScreen onto stdout and exit with errorlevel.
   error:
   	Emit the supplied message onto stderr and exit with errorlevel.
   	Preceed the message with the application name and E_error.
   fatal:
   	Equals to error, but preceed with the application name and
   	E_fatal.
   warning:
   	Emit the supplied message onto stderr and preceed the message
   	with the application name and E_warning.
   informative:
   	Emit the supplied message onto stderr and preceed the message
   	with the application name.
   interactive:
   	Emit the supplied message onto stdout. If redirection test is 
   	enabled, the message is not emitted, if stdin is redirected to
   	a non-CON: device (or, by switch, to a file).
   smessage (screen):
   	Emit the supplied message onto stdout.
   message (none):
   	Emit the supplied message onto the supplied file stream.
*/

#ifdef _MICROC_
register void error(MSGID msg);
register void warning(MSGID msg); 
register void fatal(MSGID msg);
register void informative(MSGID msg);
register void interactive(MSGID msg);
register void smessage(MSGID msg);
register void message(FILE *fp, MSGID msg);

#else /* no Micro-C */
void error(MSGID msg,...); 
void warning(MSGID msg,...); 
void fatal(MSGID msg,...);
void informative(MSGID msg,...);
void interactive(MSGID msg,...);
void smessage(MSGID msg,...);
void message(FILE *fp, MSGID msg,...);

#endif

void hlpScreen(void);
	//DCL_MSG(E_hlpScreen);

/* no format string error reporting
#define Nr_E_hlpFmtStr 20
    DCL_MSG(E_hlpFmtStr);
*/

#endif
